/*
    FEARLUS model0-7: Number.m
    Copyright (C) 1999-2003  Macaulay Institute

    This file is part of FEARLUS model0-7, an agent-based model of land use
    change.

    FEARLUS model0-7 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-7 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Implementation of Number class.

*/

#import "Number.h"
#import <string.h>
#import "Interval.h"

@implementation Number

+create: aZone {
  Number *obj = [super create: aZone];

  obj->myNumber.di = [Interval create];

  return obj;
}

-setChar: (char)value {
  myNumber.n.c = value;
  type = @encode(char);
  return self;
}
-setUnsignedChar: (unsigned char)value {
  myNumber.n.uc = value;
  type = @encode(unsigned char);
  return self;
}
-setShort: (short)value {
  myNumber.n.s = value;
  type = @encode(short);
  return self;
}
-setUnsignedShort: (unsigned short)value {
  myNumber.n.us = value;
  type = @encode(unsigned short);
  return self;
}
-setInt: (int)value {
  myNumber.n.i = value;
  type = @encode(int);
  return self;
}
-setUnsigned: (unsigned)value {
  myNumber.n.u = value;
  type = @encode(unsigned);
  return self;
}
-setLong: (long)value {
  myNumber.n.l = value;
  type = @encode(long);
  return self;
}
-setUnsignedLong: (unsigned long)value {
  myNumber.n.ul = value;
  type = @encode(unsigned long);
  return self;
}
-setLongLong: (long long)value {
  myNumber.n.ll = value;
  type = @encode(long long);
  return self;
}
-setUnsignedLongLong: (unsigned long long)value {
  myNumber.n.ull = value;
  type = @encode(unsigned long long);
  return self;
}
-setIDouble: (Interval *)value {
  [myNumber.di set: value];
  type = @encode(double);
  [value drop];
  return self;
}
-(char)getChar {
  return myNumber.n.c;
}
-(unsigned char)getUnsignedChar {
  return myNumber.n.uc;
}
-(short)getShort {
  return myNumber.n.s;
}
-(unsigned short)getUnsignedShort {
  return myNumber.n.us;
}
-(int)getInt {
  return myNumber.n.i;
}
-(unsigned)getUnsigned {
  return myNumber.n.u;
}
-(long)getLong {
  return myNumber.n.l;
}
-(unsigned long)getUnsignedLong {
  return myNumber.n.ul;
}
-(long long)getLongLong {
  return myNumber.n.ll;
}
-(unsigned long long)getUnsignedLongLong {
  return myNumber.n.ull;
}
-(Interval *)getIDouble {
  return [[myNumber.di clone] ret];
}

-(char *)getType {
  return type;
}

-(SEL)getTypeSelector {
  if(type == NULL) {
    fprintf(stderr, "WARNING: Type not set\n");
    return M(getInt);
  }
  if(strcmp(type, @encode(char)) == 0) {
    return M(getChar);
  }
  else if(strcmp(type, @encode(unsigned char)) == 0) {
    return M(getUnsignedChar);
  }
  else if(strcmp(type, @encode(short)) == 0) {
    return M(getShort);
  }
  else if(strcmp(type, @encode(unsigned short)) == 0) {
    return M(getUnsignedShort);
  }
  else if(strcmp(type, @encode(int)) == 0) {
    return M(getInt);
  }
  else if(strcmp(type, @encode(unsigned)) == 0) {
    return M(getUnsigned);
  }
  else if(strcmp(type, @encode(long)) == 0) {
    return M(getLong);
  }
  else if(strcmp(type, @encode(unsigned long)) == 0) {
    return M(getUnsignedLong);
  }
  else if(strcmp(type, @encode(long long)) == 0) {
    return M(getLongLong);
  }
  else if(strcmp(type, @encode(unsigned long long)) == 0) {
    return M(getUnsignedLongLong);
  }
  else if(strcmp(type, @encode(double)) == 0) {
    return M(getIDouble);
  }
  else {
    fprintf(stderr, "WARNING: Type not set\n");
    return M(getInt);
  }
}

-loadFromFile: (FILE *)fp {
  char buf[100];

  fscanf(fp, "%99s", buf);	// Read in the type
  if(strcmp(buf, @encode(char)) == 0) {
    fscanf(fp, " %c", &myNumber.n.c);
    type = @encode(char);
  }
  else if(strcmp(buf, @encode(unsigned char)) == 0) {
    fscanf(fp, " %uc", &myNumber.n.u);
    type = @encode(unsigned char);
  }
  else if(strcmp(buf, @encode(short)) == 0) {
    fscanf(fp, " %hd", &myNumber.n.s);
    type = @encode(short);
  }
  else if(strcmp(buf, @encode(unsigned short)) == 0) {
    fscanf(fp, " %hu", &myNumber.n.us);
    type = @encode(unsigned short);
  }
  else if(strcmp(buf, @encode(int)) == 0) {
    fscanf(fp, " %d", &myNumber.n.i);
    type = @encode(int);
  }
  else if(strcmp(buf, @encode(unsigned)) == 0) {
    fscanf(fp, " %u", &myNumber.n.u);
    type = @encode(unsigned);
  }
  else if(strcmp(buf, @encode(long)) == 0) {
    fscanf(fp, " %ld", &myNumber.n.l);
    type = @encode(long);
  }
  else if(strcmp(buf, @encode(unsigned long)) == 0) {
    fscanf(fp, " %lu", &myNumber.n.ul);
    type = @encode(unsigned long);
  }
  else if(strcmp(buf, @encode(long long)) == 0) {
    fscanf(fp, " %lld", &myNumber.n.ll);
    type = @encode(long long);
  }
  else if(strcmp(buf, @encode(unsigned long long)) == 0) {
    fscanf(fp, " %llu", &myNumber.n.ull);
    type = @encode(unsigned long long);
  }
  else if(strcmp(buf, @encode(double)) == 0) {
    [myNumber.di bufread: fp];
    type = @encode(double);
  }
  else {
    fprintf(stderr, "WARNING: Unrecognised type: %s\n", buf);
    return nil;
  }
  return self;
}

-saveToFile: (FILE *)fp {
  fprintf(fp, "%s", type);
  if(strcmp(type, @encode(char)) == 0) {
    fprintf(fp, " %c", myNumber.n.c);
  }
  else if(strcmp(type, @encode(unsigned char)) == 0) {
    fprintf(fp, " %uc", myNumber.n.uc);
  }
  else if(strcmp(type, @encode(short)) == 0) {
    fprintf(fp, " %hd", myNumber.n.s);
  }
  else if(strcmp(type, @encode(unsigned short)) == 0) {
    fprintf(fp, " %hu", myNumber.n.us);
  }
  else if(strcmp(type, @encode(int)) == 0) {
    fprintf(fp, " %d", myNumber.n.i);
  }
  else if(strcmp(type, @encode(unsigned)) == 0) {
    fprintf(fp, " %u", myNumber.n.u);
  }
  else if(strcmp(type, @encode(long)) == 0) {
    fprintf(fp, " %ld", myNumber.n.l);
  }
  else if(strcmp(type, @encode(unsigned long)) == 0) {
    fprintf(fp, " %lu", myNumber.n.ul);
  }
  else if(strcmp(type, @encode(long long)) == 0) {
    fprintf(fp, " %lld", myNumber.n.ll);
  }
  else if(strcmp(type, @encode(unsigned long long)) == 0) {
    fprintf(fp, " %llu", myNumber.n.ull);
  }
  else if(strcmp(type, @encode(double)) == 0) {
    fprintf(fp, " ");
    [myNumber.di bufwrite: fp];
  }
  return self;
}

-print {
  return [self saveToFile: stdout];
}

-(void)drop {
  [myNumber.di drop];
  [super drop];
}

@end

